(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
	typeof define === 'function' && define.amd ? define(['exports'], factory) :
	(factory((global.Realt = global.Realt || {})));
}(this, (function (exports) { 'use strict';

/**
 * This method returns the first argument it receives.
 *
 * @static
 * @since 0.1.0
 * @memberOf _
 * @category Util
 * @param {*} value Any value.
 * @returns {*} Returns `value`.
 * @example
 *
 * var object = { 'a': 1 };
 *
 * console.log(_.identity(object) === object);
 * // => true
 */
function identity(value) {
  return value;
}

var identity_1 = identity;

var commonjsGlobal = typeof window !== 'undefined' ? window : typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : {};



function unwrapExports (x) {
	return x && x.__esModule ? x['default'] : x;
}

function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

/** Detect free variable `global` from Node.js. */
var freeGlobal = typeof commonjsGlobal == 'object' && commonjsGlobal && commonjsGlobal.Object === Object && commonjsGlobal;

var _freeGlobal = freeGlobal;

/** Detect free variable `self`. */
var freeSelf = typeof self == 'object' && self && self.Object === Object && self;

/** Used as a reference to the global object. */
var root = _freeGlobal || freeSelf || Function('return this')();

var _root = root;

/** Built-in value references. */
var Symbol$1 = _root.Symbol;

var _Symbol = Symbol$1;

/** Used for built-in method references. */
var objectProto = Object.prototype;

/** Used to check objects for own properties. */
var hasOwnProperty = objectProto.hasOwnProperty;

/**
 * Used to resolve the
 * [`toStringTag`](http://ecma-international.org/ecma-262/7.0/#sec-object.prototype.tostring)
 * of values.
 */
var nativeObjectToString = objectProto.toString;

/** Built-in value references. */
var symToStringTag$1 = _Symbol ? _Symbol.toStringTag : undefined;

/**
 * A specialized version of `baseGetTag` which ignores `Symbol.toStringTag` values.
 *
 * @private
 * @param {*} value The value to query.
 * @returns {string} Returns the raw `toStringTag`.
 */
function getRawTag(value) {
  var isOwn = hasOwnProperty.call(value, symToStringTag$1),
      tag = value[symToStringTag$1];

  try {
    value[symToStringTag$1] = undefined;
    var unmasked = true;
  } catch (e) {}

  var result = nativeObjectToString.call(value);
  if (unmasked) {
    if (isOwn) {
      value[symToStringTag$1] = tag;
    } else {
      delete value[symToStringTag$1];
    }
  }
  return result;
}

var _getRawTag = getRawTag;

/** Used for built-in method references. */
var objectProto$1 = Object.prototype;

/**
 * Used to resolve the
 * [`toStringTag`](http://ecma-international.org/ecma-262/7.0/#sec-object.prototype.tostring)
 * of values.
 */
var nativeObjectToString$1 = objectProto$1.toString;

/**
 * Converts `value` to a string using `Object.prototype.toString`.
 *
 * @private
 * @param {*} value The value to convert.
 * @returns {string} Returns the converted string.
 */
function objectToString(value) {
  return nativeObjectToString$1.call(value);
}

var _objectToString = objectToString;

/** `Object#toString` result references. */
var nullTag = '[object Null]';
var undefinedTag = '[object Undefined]';

/** Built-in value references. */
var symToStringTag = _Symbol ? _Symbol.toStringTag : undefined;

/**
 * The base implementation of `getTag` without fallbacks for buggy environments.
 *
 * @private
 * @param {*} value The value to query.
 * @returns {string} Returns the `toStringTag`.
 */
function baseGetTag(value) {
  if (value == null) {
    return value === undefined ? undefinedTag : nullTag;
  }
  return (symToStringTag && symToStringTag in Object(value))
    ? _getRawTag(value)
    : _objectToString(value);
}

var _baseGetTag = baseGetTag;

/**
 * Checks if `value` is the
 * [language type](http://www.ecma-international.org/ecma-262/7.0/#sec-ecmascript-language-types)
 * of `Object`. (e.g. arrays, functions, objects, regexes, `new Number(0)`, and `new String('')`)
 *
 * @static
 * @memberOf _
 * @since 0.1.0
 * @category Lang
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is an object, else `false`.
 * @example
 *
 * _.isObject({});
 * // => true
 *
 * _.isObject([1, 2, 3]);
 * // => true
 *
 * _.isObject(_.noop);
 * // => true
 *
 * _.isObject(null);
 * // => false
 */
function isObject(value) {
  var type = typeof value;
  return value != null && (type == 'object' || type == 'function');
}

var isObject_1 = isObject;

/** `Object#toString` result references. */
var asyncTag = '[object AsyncFunction]';
var funcTag = '[object Function]';
var genTag = '[object GeneratorFunction]';
var proxyTag = '[object Proxy]';

/**
 * Checks if `value` is classified as a `Function` object.
 *
 * @static
 * @memberOf _
 * @since 0.1.0
 * @category Lang
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is a function, else `false`.
 * @example
 *
 * _.isFunction(_);
 * // => true
 *
 * _.isFunction(/abc/);
 * // => false
 */
function isFunction(value) {
  if (!isObject_1(value)) {
    return false;
  }
  // The use of `Object#toString` avoids issues with the `typeof` operator
  // in Safari 9 which returns 'object' for typed arrays and other constructors.
  var tag = _baseGetTag(value);
  return tag == funcTag || tag == genTag || tag == asyncTag || tag == proxyTag;
}

var isFunction_1 = isFunction;

var defineProperty = function (obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
};

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};



































var toConsumableArray = function (arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i];

    return arr2;
  } else {
    return Array.from(arr);
  }
};

var getInternalMethods = (function (Class) {
  return Object.getOwnPropertyNames(Class.prototype).reduce(function (object, methodName) {
    return methodName !== 'constructor' ? _extends({}, object, defineProperty({}, methodName, Class.prototype[methodName])) : object;
  }, {});
});

function createInstance(Actions) {
  if (!isFunction_1(Actions)) return Actions;

  var actions = getInternalMethods(Actions);

  /* eslint-disable no-param-reassign */

  /**
   * Creates actions by their names
   * @param {...string} actionNames
   */
  Actions.prototype.generate = function generate() {
    for (var _len = arguments.length, actionNames = Array(_len), _key = 0; _key < _len; _key++) {
      actionNames[_key] = arguments[_key];
    }

    actionNames.forEach(function (actionName) {
      actions[actionName] = identity_1;
    });
  };

  /* eslint-enable no-param-reassign */

  return _extends({}, actions, new Actions());
}

/**
 * Checks if `value` is classified as an `Array` object.
 *
 * @static
 * @memberOf _
 * @since 0.1.0
 * @category Lang
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is an array, else `false`.
 * @example
 *
 * _.isArray([1, 2, 3]);
 * // => true
 *
 * _.isArray(document.body.children);
 * // => false
 *
 * _.isArray('abc');
 * // => false
 *
 * _.isArray(_.noop);
 * // => false
 */
var isArray = Array.isArray;

var isArray_1 = isArray;

/**
 * Checks if `value` is object-like. A value is object-like if it's not `null`
 * and has a `typeof` result of "object".
 *
 * @static
 * @memberOf _
 * @since 4.0.0
 * @category Lang
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is object-like, else `false`.
 * @example
 *
 * _.isObjectLike({});
 * // => true
 *
 * _.isObjectLike([1, 2, 3]);
 * // => true
 *
 * _.isObjectLike(_.noop);
 * // => false
 *
 * _.isObjectLike(null);
 * // => false
 */
function isObjectLike(value) {
  return value != null && typeof value == 'object';
}

var isObjectLike_1 = isObjectLike;

/** `Object#toString` result references. */
var stringTag = '[object String]';

/**
 * Checks if `value` is classified as a `String` primitive or object.
 *
 * @static
 * @since 0.1.0
 * @memberOf _
 * @category Lang
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is a string, else `false`.
 * @example
 *
 * _.isString('abc');
 * // => true
 *
 * _.isString(1);
 * // => false
 */
function isString(value) {
  return typeof value == 'string' ||
    (!isArray_1(value) && isObjectLike_1(value) && _baseGetTag(value) == stringTag);
}

var isString_1 = isString;

var getTypePrefix = (function (prefix) {
  return isString_1(prefix) ? prefix.toUpperCase() + '_' : '';
});

var toConstant = (function (string) {
	return string.replace(/[a-z]([A-Z])/g, function (i) {
		return i[0] + "_" + i[1].toLowerCase();
	}).toUpperCase();
});

function makeAction(prefix, name, payload) {
  var type = '' + getTypePrefix(prefix) + toConstant(name);
  var finalPayload = isFunction_1(payload) ? payload : identity_1;

  var action = function action() {
    return {
      type: type, payload: finalPayload.apply(undefined, arguments)
    };
  };

  action.toString = function () {
    return type;
  };

  return action;
}

/**
 * Create Redux actions from Class or object
 * @param {Class|Object} Actions
 * @param {String} [typePrefix]
 * @return {Object} Action creators
 */
function createActions(Actions, typePrefix) {
  var actions = createInstance(Actions);

  return Object.keys(actions).reduce(function (result, name) {
    return _extends({}, result, defineProperty({}, name, makeAction(typePrefix, name, actions[name])));
  }, {});
}

var index = createCommonjsModule(function (module, exports) {
"use strict";

exports.__esModule = true;
exports["default"] = reduceReducers;

function reduceReducers() {
  for (var _len = arguments.length, reducers = Array(_len), _key = 0; _key < _len; _key++) {
    reducers[_key] = arguments[_key];
  }

  return function (previous, current) {
    return reducers.reduce(function (p, r) {
      return r(p, current);
    }, previous);
  };
}

module.exports = exports["default"];
});

var reduceReducers = unwrapExports(index);

function makeActionHandler(reducerHandler, prefix, actionType, initialState) {
  var actionCreatorType = '' + getTypePrefix(prefix) + actionType;

  return function () {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
    var action = arguments[1];
    return actionCreatorType === action.type ? reducerHandler(state, action.payload) : state;
  };
}

function createInstanceByClass(ReducerClass, typePrefix) {
  var reducers = [];

  /* eslint-disable no-param-reassign */

  /**
   * Attaches reducer's handler function to specific action type
   * @param {Function} reducerHandler
   * @param {Object} actionCreator
   */
  ReducerClass.prototype.bindHandler = function bindHandler(reducerHandler, actionCreator) {
    reducers.push(makeActionHandler(reducerHandler, typePrefix, actionCreator.toString(), this.initialState));
  };

  /**
   * Binds actionCreator for specific reducer handler function
   * @param {Object} actionCreator
   * @param {Function} reducerHandler
   */
  ReducerClass.prototype.bindAction = function bindAction(actionCreator, reducerHandler) {
    this.bindHandler(reducerHandler, actionCreator);
  };

  /* eslint-enable no-param-reassign */

  var reducer = new ReducerClass();

  return { reducers: reducers, initialState: reducer.initialState };
}

function createInstanceByObject(reducerObject, typePrefix) {
  var reducers = [];
  var prefixes = ['handle', 'reducerFor'];
  var initialState = reducerObject.initialState;

  var actionType = null;
  var reducerHandler = null;

  Object.keys(reducerObject).forEach(function (reducer) {
    actionType = reducer;
    prefixes.forEach(function (prefix) {
      if (reducer.includes(prefix)) actionType = reducer.replace(prefix, '');
    });

    actionType = toConstant(actionType);
    reducerHandler = reducerObject[reducer];

    reducers.push(makeActionHandler(reducerHandler, typePrefix, actionType, initialState));
  });

  return { reducers: reducers, initialState: initialState };
}

function createInstance$1(Reducer, typePrefix) {
  if (isFunction_1(Reducer)) return createInstanceByClass(Reducer, typePrefix);

  return createInstanceByObject(Reducer, typePrefix);
}

/**
 * Create Redux reducer from Class
 * @param {Class|Object} Reducer
 * @param {Object} [initialState]
 * @param {String} [typePrefix]
 * @return {Function} Reducer
 */
function createReducer(Reducer, initialState, typePrefix) {
  var _createInstance = createInstance$1(Reducer, typePrefix),
      reducers = _createInstance.reducers,
      reducerInitialState = _createInstance.initialState;

  var reducer = reduceReducers.apply(undefined, toConsumableArray(reducers));

  return function () {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState || reducerInitialState;
    var action = arguments[1];
    return reducer(state, action);
  };
}

exports.createActions = createActions;
exports.createReducer = createReducer;

Object.defineProperty(exports, '__esModule', { value: true });

})));
